﻿<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:msxsl="urn:schemas-microsoft-com:xslt" exclude-result-prefixes="msxsl"
>
  <xsl:output method="html" indent="yes" omit-xml-declaration="yes" />
  <xsl:param name="MachineName">oMachine</xsl:param>
  <xsl:variable name="LineFeed"><xsl:text>&#10;&#13;</xsl:text></xsl:variable>
  
  <xsl:template match="GIMachine">
    var <xsl:value-of select="$MachineName"/> = new GIMachine("<xsl:value-of select="Name"/>");
    <xsl:for-each select="GIMWordState">
      <xsl:variable name="stateIndex" select="position()-1"/>
      <xsl:call-template name="GenerateCodeForMachine">
        <xsl:with-param name="GIState" select="." />
        <xsl:with-param name="ParentVariableName" select="$MachineName" />
        <xsl:with-param name="StateIndex" select="$stateIndex" />
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="GenerateCodeForState">
    <xsl:param name="GIState" />
    <xsl:param name="ParentVariableName" />
    <xsl:param name="StateIndex" />
    <xsl:variable name="ThisStateVariableName"><xsl:value-of select="$ParentVariableName"/>.NextStates[<xsl:value-of select="$StateIndex"/>]</xsl:variable>
    <xsl:variable name="JSFriendlyPattern">
      <xsl:call-template name="string-replace-all">
        <xsl:with-param name="text" select="$GIState/Pattern" />
        <xsl:with-param name="replace">\</xsl:with-param>
        <xsl:with-param name="by" >\\</xsl:with-param>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:value-of select="$ThisStateVariableName"/> = new GIMachineWordState("<xsl:value-of select="$JSFriendlyPattern" disable-output-escaping="yes" />", <xsl:value-of select="$GIState/IsLiteral" disable-output-escaping="yes" />,<xsl:value-of select="$GIState/ID" disable-output-escaping="yes" />,"<xsl:value-of select="$GIState/Name" disable-output-escaping="yes" />", <xsl:value-of select="$GIState/IsAcceptable" disable-output-escaping="yes" />);

    <xsl:for-each select="$GIState/GIMWordState">
      <xsl:variable name="stateIndex" select="position()-1"/>
      <xsl:call-template name="GenerateCodeForState">
        <xsl:with-param name="GIState" select="." />
        <xsl:with-param name="ParentVariableName" select="$ThisStateVariableName" />
        <xsl:with-param name="StateIndex" select="$stateIndex" />
      </xsl:call-template>
    </xsl:for-each>
    
  </xsl:template>


  <xsl:template name="GenerateCodeForMachine">
    <xsl:param name="GIState" />
    <xsl:param name="ParentVariableName" />
    <xsl:param name="StateIndex" />
    <xsl:variable name="ThisStateVariableName">
      <xsl:value-of select="$ParentVariableName"/>.States[<xsl:value-of select="$StateIndex"/>]
    </xsl:variable>
    <xsl:variable name="JSFriendlyPattern">
      <xsl:call-template name="string-replace-all">
        <xsl:with-param name="text" select="$GIState/Pattern" />
        <xsl:with-param name="replace">\</xsl:with-param>
        <xsl:with-param name="by" >\\</xsl:with-param>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:value-of select="$ThisStateVariableName"/> = new GIMachineWordState("<xsl:value-of select="$JSFriendlyPattern" disable-output-escaping="yes" />", <xsl:value-of select="$GIState/IsLiteral" disable-output-escaping="yes" />,<xsl:value-of select="$GIState/ID" disable-output-escaping="yes" />,"<xsl:value-of select="$GIState/Name" disable-output-escaping="yes" />", <xsl:value-of select="$GIState/IsAcceptable" disable-output-escaping="yes" />);

    <xsl:for-each select="$GIState/GIMWordState">
      <xsl:variable name="stateIndex" select="position()-1"/>
      <xsl:call-template name="GenerateCodeForState">
        <xsl:with-param name="GIState" select="." />
        <xsl:with-param name="ParentVariableName" select="$ThisStateVariableName" />
        <xsl:with-param name="StateIndex" select="$stateIndex" />
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="string-replace-all">
    <xsl:param name="text" />
    <xsl:param name="replace" />
    <xsl:param name="by" />
    <xsl:choose>
      <xsl:when test="contains($text, $replace)">
        <xsl:value-of select="substring-before($text,$replace)" />
        <xsl:value-of select="$by" />
        <xsl:call-template name="string-replace-all">
          <xsl:with-param name="text"
          select="substring-after($text,$replace)" />
          <xsl:with-param name="replace" select="$replace" />
          <xsl:with-param name="by" select="$by" />
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


</xsl:stylesheet>
